/*
 * Copyright (C) 2020. Huawei Technologies Co., Ltd. All rights reserved.
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */
#include <glib.h>
#include <glib-unix.h>

#ifndef IRQBALANCE_CLIENT_H
#define IRQBALANCE_CLIENT_H

/* ERRORNO */
#define IRQBALANCE_SUCCESS 0
#define IRQBALANCE_INPUT_ILLEGAL 1
#define IRQBALANCE_CONNECT_FAIL 2
#define IRQBALANCE_SEND_FAIL 3
#define IRQBALANCE_MALLOC_FAIL 4

#define BAN_CPUS "settings cpus "
#define DATA_BUF_MAX_LEN 128
#define NR_CPUS 1024
#define CPU_LIST_MAX_LEN 4096
#define CPU_MASK_MAX_LEN 256

typedef enum irqbalance_node_type {
	IRQ_OBJ_TYPE_CPU,
	IRQ_OBJ_TYPE_CACHE,
	IRQ_OBJ_TYPE_PACKAGE,
	IRQ_OBJ_TYPE_NODE
} irqbalance_node_type_e;

typedef struct irqbalance_irq {
	int vector;
	unsigned long load;
	unsigned long diff;
	char is_banned;
	GList *assigned_to;
	int class;
} irqbalance_irq_t;

typedef struct irqbalance_cpu_node {
	irqbalance_node_type_e type;
	int number;
	unsigned long load;
	int is_powersave;
	struct irqbalance_cpu_node *parent;
	GList *children;
	GList *irqs;
	GList *cpu_list;
	char *cpu_mask;
} irqbalance_cpu_node_t;

typedef struct irqbalance_banned_irq_list {
	int irq;
	int class;
	unsigned long load;
	unsigned long diff;
	struct irqbalance_banned_irq_list *next;
} irqbalance_banned_irq_list_t;

typedef struct irqbalance_banned_irq_info {
	int irq;
	int class;
	unsigned long load;
	unsigned long diff;
} irqbalance_banned_irq_t;

typedef struct irqbalance_setup_data {
	int sleep;
	char banned_cpus[NR_CPUS + 1];
	int ban_irq_num;
	irqbalance_banned_irq_t banned_irqs[1];
} irqbalance_setup_t;

/*
 * set_ban_irqs string format:
 *    <int> <int> <int> <int> ... or NONE
 * */
int irqbalance_set_ban_irqs(const char *irqs);

/*
 * set_ban_cpus string format:
 *    00000001  or NULL
 * */
int irqbalance_set_ban_cpus(const char *cpus);

/* set sleep interval of irqbalance main loop */
int irqbalance_set_sleep_interval(int sleep);

/* get user setup info, including sleep setting, banned irqs and banned cpus info */
irqbalance_setup_t *irqbalance_get_setup_info();

/* get irqbalance stats tree */
GList *irqbalance_get_stats_info();

/* get banned cpus mask */
char *irqbalance_get_banned_cpus();

/* get banned irqs string */
char *irqbalance_get_banned_irqs();

/* set the pid of irqbalance server */
void irqbalance_set_pid(int pid);
#endif
